import { MarkdownItHeader } from '@mdit-vue/types';
export { isArray, isFunction, isString } from '@vue/shared';

/**
 * Config for `<head>` tags
 */
type HeadConfig = [HeadTagEmpty, HeadAttrsConfig] | [HeadTagNonEmpty, HeadAttrsConfig, string];
/**
 * Allowed tags in `<head>`
 *
 * @see https://developer.mozilla.org/en-US/docs/Web/HTML/Element/head
 */
type HeadTag = HeadTagNonEmpty | HeadTagEmpty;
/**
 * Non-empty tags in `<head>`
 */
type HeadTagNonEmpty = 'title' | 'style' | 'script' | 'noscript' | 'template';
/**
 * Empty tags in `<head>`
 */
type HeadTagEmpty = 'base' | 'link' | 'meta' | 'script';
/**
 * Attributes to be set for tags in `<head>`
 */
type HeadAttrsConfig = Record<string, string | boolean>;

/**
 * Locales config, a key-value object
 *
 * - Key is the locale path (prefix)
 * - Value is the locales data
 *
 * @remark suffix `Config` means this is for user config
 */
type LocaleConfig<T extends LocaleData = LocaleData> = Record<string, Partial<T>>;
/**
 * Locales data
 */
type LocaleData = Record<never, never>;

/**
 * Base type of vuepress page
 */
type PageBase<ExtraPageFrontmatter extends Record<any, any> = Record<string, unknown>> = {
    /**
     * Identifier of the page
     *
     * Will also be used as the component name
     *
     * @example 'v-foobar'
     */
    key: string;
    /**
     * Route path of the page
     *
     * Firstly inferred from the file path
     *
     * Might be overridden by permalink
     *
     * @example '/guide/index.html'
     * @example '/2020/02/02/hello-world.html'
     */
    path: string;
    /**
     * Title of the page
     */
    title: string;
    /**
     * Language of the page
     */
    lang: string;
    /**
     * Front matter of the page
     */
    frontmatter: PageFrontmatter<ExtraPageFrontmatter>;
    /**
     * Headers of the page
     */
    headers: PageHeader[];
};
/**
 * Vuepress page data
 */
type PageData<ExtraPageData extends Record<any, any> = Record<never, never>, ExtraPageFrontmatter extends Record<any, any> = Record<string, unknown>> = PageBase<ExtraPageFrontmatter> & ExtraPageData;
/**
 * Vuepress page frontmatter
 *
 * Notice that frontmatter is parsed from yaml or other languages,
 * so we cannot guarantee the type safety
 */
type PageFrontmatter<T extends Record<any, any> = Record<string, unknown>> = Partial<T> & {
    date?: string | Date;
    description?: string;
    head?: HeadConfig[];
    lang?: string;
    layout?: string;
    permalink?: string;
    permalinkPattern?: string | null;
    routeMeta?: Record<string, unknown>;
    title?: string;
};
/**
 * Vuepress page header
 */
type PageHeader = MarkdownItHeader;

/**
 * Vuepress site data
 */
interface SiteData extends SiteLocaleData {
    /**
     * The base URL the site will be deployed at
     *
     * It should always start and end with a slash
     *
     * @default '/'
     */
    base: '/' | `/${string}/`;
    /**
     * Specify locales for i18n support
     *
     * It will override the root-level site data in different subpath
     *
     * @example
     * {
     *   '/en/': {
     *     lang: 'en-US',
     *     title: 'Hello',
     *     description: 'This will take effect under /en/ subpath',
     *   },
     *   '/zh/': {
     *     lang: 'zh-CN',
     *     title: '你好',
     *     description: '它将会在 /zh/ 子路径下生效',
     *   }
     * }
     */
    locales: SiteLocaleConfig;
}
/**
 * Locales data of vuepress site
 *
 * If they are set in the root of site data, they will be used
 * as the default value
 *
 * If they are set in the `locales` of site data, they will be
 * used for specific locale
 */
interface SiteLocaleData {
    /**
     * Language for the site
     *
     * @default 'en-US'
     */
    lang: string;
    /**
     * Title for the site
     *
     * @default ''
     */
    title: string;
    /**
     * Description for the site
     *
     * @default ''
     */
    description: string;
    /**
     * Head config
     *
     * Descibe the tags to be appended into the `<head>` tag
     *
     * @default []
     *
     * @example ['link', { rel: 'icon', href: '/logo.png' }]
     * @example ['style', { type: 'text/css' }, 'p { color: red; }']
     */
    head: HeadConfig[];
}
/**
 * Site locale config
 */
type SiteLocaleConfig = LocaleConfig<SiteLocaleData>;

/**
 * Context for SSR
 */
interface VuepressSSRContext {
    lang: string;
    head: HeadConfig[];
}

/**
 * Dedupe head config with identifier
 *
 * Items that appear earlier have higher priority
 */
declare const dedupeHead: (head: HeadConfig[]) => HeadConfig[];

/**
 * Ensure a url string to have leading slash /
 */
declare const ensureLeadingSlash: (str: string) => string;

/**
 * Ensure a url string to have ending slash /
 */
declare const ensureEndingSlash: (str: string) => string;

/**
 * Format a date string to `yyyy-MM-dd`
 */
declare const formatDateString: (str: string, defaultDateString?: string) => string;

/**
 * Determine a link is external or not
 */
declare const isLinkExternal: (link: string, base?: string) => boolean;

/**
 * Determine a link is ftp link or not
 */
declare const isLinkFtp: (link: string) => boolean;

/**
 * Determine a link is http link or not
 *
 * - http://github.com
 * - https://github.com
 * - //github.com
 */
declare const isLinkHttp: (link: string) => boolean;

/**
 * Determine a link is a mailto: address or not
 */
declare const isLinkMailto: (link: string) => boolean;

/**
 * Determine a link is a tel: address or not
 */
declare const isLinkTel: (link: string) => boolean;

/**
 * Check if a value is plain object, with generic type support
 */
declare const isPlainObject: <T extends Record<any, any> = Record<any, any>>(val: unknown) => val is T;

/**
 * Omit properties from an object
 */
declare const omit: <T extends Record<string, unknown>, U extends string[]>(obj: T, ...keys: U) => Omit<T, U[number]>;

/**
 * Remove ending slash / from a string
 */
declare const removeEndingSlash: (str: string) => string;

/**
 * Remove leading slash / from a string
 */
declare const removeLeadingSlash: (str: string) => string;

/**
 * Resolve identifier of a tag, to avoid duplicated tags in `<head>`
 */
declare const resolveHeadIdentifier: ([tag, attrs, content,]: HeadConfig) => string;

/**
 * Resolve the matched locale path of route path
 */
declare const resolveLocalePath: (locales: LocaleConfig, routePath: string) => string;

declare const resolveRoutePathFromUrl: (url: string, base?: string) => string;

export { HeadAttrsConfig, HeadConfig, HeadTag, HeadTagEmpty, HeadTagNonEmpty, LocaleConfig, LocaleData, PageBase, PageData, PageFrontmatter, PageHeader, SiteData, SiteLocaleConfig, SiteLocaleData, VuepressSSRContext, dedupeHead, ensureEndingSlash, ensureLeadingSlash, formatDateString, isLinkExternal, isLinkFtp, isLinkHttp, isLinkMailto, isLinkTel, isPlainObject, omit, removeEndingSlash, removeLeadingSlash, resolveHeadIdentifier, resolveLocalePath, resolveRoutePathFromUrl };
